from kivy.uix.screenmanager import Screen
from kivy.uix.gridlayout import GridLayout
from kivy.uix.label import Label
from kivy.uix.button import Button
from kivy.properties import NumericProperty

import background

FADEIN, IDLE, FADEOUT = xrange(3)


class EscLabel(Label):
    pass


class TextLabel(Label):
    font_scale = NumericProperty(.09)


class ResponseButton(Button):
    def __init__(self, app):
        self.app = app
        self.response = None
        Button.__init__(self)

    def set_response(self, response):
        self.text = response.value
        self.response = response

    def on_release(self):
        if self.enabled:
            self.app.story_manager.on_response(self.response)


class ResponseGrid(GridLayout):
    def __init__(self, app):
        GridLayout.__init__(self)

        self.buttons = [ResponseButton(app) for _ in xrange(4)]
        for b in self.buttons:
            self.add_widget(b)

    def populate(self, responses):
        for i, b in enumerate(self.buttons):
            if i < len(responses):
                b.set_response(responses[i])
                b.enabled = True
                b.opacity = 1
            else:
                b.enabled = False
                b.opacity = 0


class DialogueScreen(Screen):
    fast_fade = 10
    slow_fade = 1

    def __init__(self, app):
        Screen.__init__(self, name='dialogue')

        self.opacity = 0

        self.background = background.Background()
        self.text_label = TextLabel()
        self.response_grid = ResponseGrid(app)

        self.add_widget(self.background)
        self.add_widget(self.text_label)
        self.add_widget(self.response_grid)
        self.add_widget(EscLabel())

        self.state = 0
        self.timer = 0
        self.dialogue = None
        self.next_background = None
        self.next_font = None

    def show_dialogue(self, d):
        if self.dialogue:
            self.dialogue = d
            self.set_state(FADEOUT)
        else:
            self.dialogue = d
            self.set_state(FADEIN)

    def set_state(self, state):
        if state == FADEIN:
            self.text_label.text = self.dialogue.get_text()
            self.response_grid.populate(self.dialogue.get_responses())
            if self.next_background is None and self.next_font is None:
                self.fade_speed = self.fast_fade

            else:
                self.fade_speed = self.slow_fade

                if self.next_background is not None:
                    self.background.source = self.next_background
                    self.background.update(self.size)
                    self.next_background = None
                if self.next_font is not None:
                    self.text_label.font_name = self.next_font[0]
                    self.text_label.font_scale = self.next_font[1]
                    self.next_font = None

            self.timer = 1
        elif state == FADEOUT:
            if (self.next_background is not None) or (self.next_font is not None) or (self.dialogue is None):
                self.fade_speed = self.slow_fade
            else:
                self.fade_speed = self.fast_fade
            self.timer = 1
        elif state == IDLE:
            pass
        self.state = state

    def on_touch_down(self, touch):
        if self.state == IDLE:
            Screen.on_touch_down(self, touch)
        else:
            return True

    def on_size(self, *args):
        self.background.update(self.size)

    def set_background(self, filename):
        self.next_background = filename

    def set_font(self, filename, scale):
        self.next_font = filename, scale

    def update(self, dt):
        if self.state != IDLE:
            self.timer -= self.fade_speed * dt

            if self.state == FADEIN:
                self.opacity = 1 - self.timer
                if self.timer <= 0:
                    self.set_state(IDLE)

            elif self.state == FADEOUT:
                self.opacity = self.timer

                if self.timer <= 0:
                    self.set_state(FADEIN)
