import pymunk

NONE, ENEMY, PLAYER, WALL, BULLET = [2 ** i for i in xrange(5)]

WALL_FILTER = pymunk.ShapeFilter(categories=WALL)
BULLET_FILTER = pymunk.ShapeFilter(categories=BULLET)
ENEMY_FILTER = pymunk.ShapeFilter(categories=ENEMY)
PLAYER_FILTER = pymunk.ShapeFilter(categories=PLAYER)


class PhysicsManager:
    def __init__(self, app):
        self.app = app
        self.space = pymunk.Space()
        handler = self.space.add_collision_handler(BULLET, PLAYER)
        handler.begin = self.bullet_player
        handler = self.space.add_collision_handler(BULLET, ENEMY)
        handler.begin = self.bullet_enemy
        handler = self.space.add_collision_handler(BULLET, WALL)
        handler.begin = self.bullet_wall

    def bullet_player(self, arbiter, space, data):
        bullet_collider, player_collider = arbiter.shapes
        self.app.game_manager.remove_bullet(bullet_collider.body.parent)
        player_collider.body.parent.damage(1)
        return True

    def bullet_enemy(self, arbiter, space, data):
        bullet_collider, enemy_collider = arbiter.shapes
        self.app.game_manager.remove_bullet(bullet_collider.body.parent)
        enemy_collider.body.parent.damage(1)
        return True

    def bullet_wall(self, arbiter, space, data):
        bullet_collider, wall_collider = arbiter.shapes
        bullet = bullet_collider.body.parent
        if bullet.bounce_count >= bullet.max_bounces:
            self.app.game_manager.remove_bullet(bullet)
        else:
            bullet.bounce_count += 1
        return True

    def clear_static(self):
        self.space.remove(self.space.static_body.shapes)
        self.space.step(0)

    def load_colliders(self, scene):
        for obj in scene.objects.itervalues():
            amt = sum([i[1] for i in obj.vertex_format])
            for tri in [obj.vertices[i:i + amt * 3] for i in xrange(0, len(obj.vertices), amt * 3)]:
                points = []
                for point in [tri[i:i + 3] for i in xrange(0, len(tri), amt)]:
                    if point[1] == 0.0:
                        points.append((point[0], point[2]))
                if len(points) == 2:
                    collider = pymunk.Segment(self.space.static_body, points[0], points[1], .1)
                    collider.elasticity = 1
                    collider.friction = 1
                    collider.collision_type = WALL
                    collider.filter = WALL_FILTER
                    self.space.add(collider)

    def update(self, dt):
        self.space.step(dt)
