import math

import robot, physics, game_manager

DIE, ATTACK, WALKING = xrange(3)


class Player(robot.Robot):
    speed = 1.2
    fire_rate = .25

    collision_type = physics.PLAYER

    def __init__(self, app):
        robot.Robot.__init__(self, app, (1, 0, 0))

        self.set_state(WALKING)
        self.input_vector = [0, 0]

    def on_axis(self, axis, value):
        self.input_vector[axis] = min(1, max(-1, self.input_vector[axis] + value))

    def set_state(self, state):
        if state == ATTACK:
            self.state_timer = self.fire_rate * 2
            self.set_animation(robot.AIM_ANIM)
        elif state == WALKING:
            self.set_animation(robot.WALK_ANIM)
        elif state == DIE:
            self.set_animation(robot.DEATH_ANIM)
            self.app.game_manager.set_state(game_manager.FAIL)
        self.state = state

    def respawn(self, x, z):
        self.set_position(x, 0, z)
        self.set_state(WALKING)
        self.health = self.max_health

    def attack(self):
        if self.health > 0:
            self.set_state(ATTACK)

    def stop_attack(self):
        if self.health > 0:
            self.set_state(WALKING)

    def update(self, dt, factor=1):
        self.do_animation(dt)
        if self.state != DIE:
            if any(self.input_vector):
                r = self.app.scene.cam_yaw.angle / 57.3

                cos = math.cos(r)
                sin = math.sin(r)

                ix = self.input_vector[0]
                iy = self.input_vector[1]

                fx = ix * cos + iy * sin
                fy = ix * -sin + iy * cos

                if all(self.input_vector):
                    speed = self.speed * .7
                else:
                    speed = self.speed

                self.body.apply_impulse_at_local_point((fx * speed, -fy * speed))

                if self.state == WALKING:
                    self.face_velocity(dt)

            robot.Robot.update(self, dt)
            if self.state == ATTACK:
                arm_index = int(self.state_timer > self.fire_rate)
                self.state_timer -= dt
                if self.state_timer < arm_index * self.fire_rate:
                    self.fire(arm_index)
                    if arm_index is 0:
                        self.state_timer += self.fire_rate * 2

                self.target_angle = 90 - self.app.scene.cam_yaw.angle

            self.head_yaw.angle = -self.app.scene.cam_yaw.angle - self.rotation.angle
            self.head_pitch.angle = -self.app.scene.cam_pitch.angle

            self.position.x = self.body.position.x
            self.position.z = self.body.position.y

            self.app.scene.set_cam_pos(self.position.x,
                                       4,
                                       self.position.z)
