import operator

equals = operator.eq
not_equals = operator.ne
less_than = operator.lt
greater_than = operator.gt


class CharacterStory:
    def __init__(self, dialogues, start_name):
        self.dialogues = {d.name: d for d in dialogues}
        self.start_name = start_name

        self.current_dialogue = None

    def set_dialogue(self, dialogue_name):
        if self.current_dialogue is not None:
            self.current_dialogue.on_leave()
            self.current_dialogue = None

        if dialogue_name in self.dialogues:
            self.current_dialogue = self.dialogues[dialogue_name]
            self.current_dialogue.on_enter()
            return self.current_dialogue
        else:
            return None


class Dialogue:
    def __init__(self, name, texts, responses, enter_actions=[], leave_actions=[]):
        self.name = name
        self.texts = texts
        self.responses = responses
        self.enter_actions = enter_actions
        self.leave_actions = leave_actions

    def get_text(self):
        for text in self.texts:
            if text.passes_criteria():
                return text.value
        return 'I don\'t know what to say :/'

    def get_responses(self):
        return [r for r in self.responses if r.passes_criteria()]

    def on_enter(self):
        for a in self.enter_actions:
            a.do_action()

    def on_leave(self):
        for a in self.leave_actions:
            a.do_action()


class Value:
    def __init__(self, value, criteria=[]):
        self.value = value
        self.criteria = criteria

    def passes_criteria(self):
        for c in self.criteria:
            if not c.is_true():
                return False
        return True


class Response(Value):
    def __init__(self, text, links=[], actions=[], criteria=[]):
        Value.__init__(self, text, criteria)
        self.actions = actions
        self.links = links

    def do_actions(self):
        for a in self.actions:
            a.do_action()

    def get_link(self):
        for l in self.links:
            if l.passes_criteria():
                return l.value
        return None


class Criterion:
    def __init__(self, func, args, cmp_func=equals, test_value=True):
        self.func = func
        self.cmp_func = cmp_func
        self.args = args
        self.test_value = test_value

    def is_true(self):
        return self.cmp_func(self.func(*self.args), self.test_value)


class Action:
    def __init__(self, func, args):
        self.func = func
        self.args = args

    def do_action(self):
        self.func(*self.args)
